// SPDX-FileCopyrightText: 1989-1999 Free Software Foundation, Inc.
// SPDX-License-Identifier: GPL-2.0-or-later

/* Defs for interface to demanglers.
   Copyright 1992, 1995, 1996 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#if !defined(DEMANGLE_H)
#define DEMANGLE_H

#define PARAMS(x) x

/* Options passed to cplus_demangle (in 2nd parameter). */

#define DMGL_NO_OPTS 0 /* For readability... */
#define DMGL_PARAMS  (1 << 0) /* Include function args */
#define DMGL_ANSI    (1 << 1) /* Include const, volatile, etc */
#define DMGL_JAVA    (1 << 2) /* Demangle as Java rather than C++. */

#define DMGL_AUTO  (1 << 8)
#define DMGL_GNU   (1 << 9)
#define DMGL_LUCID (1 << 10)
#define DMGL_ARM   (1 << 11)
#define DMGL_HP    (1 << 12) /* For the HP aCC compiler; same as ARM \
				except for template arguments, etc. */
#define DMGL_EDG (1 << 13)

/* If none of these are set, use 'current_demangling_style' as the default. */
#define DMGL_STYLE_MASK (DMGL_AUTO | DMGL_GNU | DMGL_LUCID | DMGL_ARM | DMGL_HP | DMGL_EDG)

/* Enumeration of possible demangling styles.

   Lucid and ARM styles are still kept logically distinct, even though
   they now both behave identically.  The resulting style is actual the
   union of both.  I.E. either style recognizes both "__pt__" and "__rf__"
   for operator "->", even though the first is lucid style and the second
   is ARM style. (FIXME?) */

extern enum demangling_styles {
	unknown_demangling = 0,
	auto_demangling = DMGL_AUTO,
	gnu_demangling = DMGL_GNU,
	lucid_demangling = DMGL_LUCID,
	arm_demangling = DMGL_ARM,
	hp_demangling = DMGL_HP,
	edg_demangling = DMGL_EDG
} current_demangling_style;

/* Define string names for the various demangling styles. */

#define AUTO_DEMANGLING_STYLE_STRING  "auto"
#define GNU_DEMANGLING_STYLE_STRING   "gnu"
#define LUCID_DEMANGLING_STYLE_STRING "lucid"
#define ARM_DEMANGLING_STYLE_STRING   "arm"
#define HP_DEMANGLING_STYLE_STRING    "hp"
#define EDG_DEMANGLING_STYLE_STRING   "edg"

/* Some macros to test what demangling style is active. */

#define CURRENT_DEMANGLING_STYLE current_demangling_style
#define AUTO_DEMANGLING          (((int)CURRENT_DEMANGLING_STYLE) & DMGL_AUTO)
#define GNU_DEMANGLING           (((int)CURRENT_DEMANGLING_STYLE) & DMGL_GNU)
#define LUCID_DEMANGLING         (((int)CURRENT_DEMANGLING_STYLE) & DMGL_LUCID)
#define ARM_DEMANGLING           (((int)CURRENT_DEMANGLING_STYLE) & DMGL_ARM)
#define HP_DEMANGLING            (((int)CURRENT_DEMANGLING_STYLE) & DMGL_HP)
#define EDG_DEMANGLING           (((int)CURRENT_DEMANGLING_STYLE) & DMGL_EDG)

extern char *
	cplus_demangle_v2 PARAMS((const char *mangled, int options));

#endif /* DEMANGLE_H */
